function x = pagerank1(G,p)
% PAGERANK1  Google's PageRank
% x = pagerank1(G,p) uses direct solution of the sparse page
% rank equations to compute the page rank for a connectivity
% matrix G with a damping factory p, (default is .85).
% See also: SURFER, PAGERANK2, PAGERANK3.

% Eliminate any self-referential links.
% Any node with no out links is temporarily linked to all nodes.
% c = column sums = in-degree.

G = G - diag(diag(G));
c = sum(G);
for j = find(c==0);
   G(:,j) = 1;
end
c = sum(G);

% Directly solve page rank equations.

[n,n] = size(G);
if nargin < 2, p = .85; end
delta = (1-p)/n;
e = ones(n,1);
I = speye(n,n);
D = sparse(diag(1./c));
x = (I - p*G*D)\(delta*e);
