function x = pagerank2(G,p)
% PAGERANK2  Google's PageRank
% x = pagerank2(G,p) uses inverse iteration to compute the page
% rank for a connectivity matrix G with a damping factory p,
% (default is .85).
% See also: SURFER, PAGERANK1, PAGERANK3.

% Eliminate any self-referential links.
% Any node with no out links is temporarily linked to all nodes.
% c = column sums = in-degree.

G = G - diag(diag(G));
c = sum(G);
for j = find(c==0);
   G(:,j) = 1;
end
c = sum(G);

% Normalized adjacency matrix

[n,n] = size(G);
if nargin < 2, p = .85; end
delta = (1-p)/n;
A = p*G/diag(c) + delta;

% One step of inverse iteration to find Markov vector, A*x = x.

warning off MATLAB:nearlySingularMatrix
e = ones(n,1)/n;
I = eye(n,n);
x = (I - A)\e;
x = x/sum(x);
warning on MATLAB:nearlySingularMatrix
