function x = pagerank3(G,p)
% PAGERANK3  Google's PageRank
% x = pagerank1(G,p) uses the power method to compute the page
% rank for a connectivity matrix G with a damping factory p,
% (default is .85).
% See also: SURFER, PAGERANK1, PAGERANK2.

% Eliminate any self-referential links.
% Any node with no out links is temporarily linked to all nodes.
% c = column sums = in-degree.

G = G - diag(diag(G));
c = sum(G);
for j = find(c==0);
   G(:,j) = 1;
end
c = sum(G);

% Normalized adjacency matrix

[n,n] = size(G);
if nargin < 2, p = .85; end
delta = (1-p)/n;
A = p*G/diag(c) + delta;

% Power method to find Markov vector, A*x = x.

x = ones(n,1)/n;
xs = zeros(n,1);
while max(abs(x - xs)) > 1.e-10
   xs = x;
   x = A*x;
end
